from typing import Optional, Any, Mapping

from kubragen.kdatahelper import KDataHelper_Volume
from kubragen.option import OptionDef, OptionDefFormat
from kubragen.options import Options


class EFKOptions(Options):
    """
    Options for the EFK builder.

    .. list-table::
        :header-rows: 1

        * - option
          - description
          - allowed types
          - default value
        * - basename
          - object names prefix
          - str
          - ```efk```
        * - namespace
          - namespace
          - str
          - ```default```
        * - config |rarr| elasticsearch_replicas
          - number of elasticsearch replicas
          - int
          - ```3```
        * - config |rarr| probes
          - whether to enable liveness / rediness probes
          - bool
          - ```True```
        * - config |rarr| kkibana_service_port
          - Kibana service port
          - int
          - ```80```
        * - config |rarr| authorization |rarr| serviceaccount_create
          - whether to create a service account
          - bool
          - ```True```
        * - config |rarr| authorization |rarr| serviceaccount_use
          - service account to use if not creating
          - str
          -
        * - config |rarr| authorization |rarr| roles_create
          - whether create roles
          - bool
          - ```True```
        * - config |rarr| authorization |rarr| roles_bind
          - whether to bind roles to service account
          - bool
          - ```True```
        * - enable |rarr| kibana
          - whether to enable the kibana deployment
          - bool
          - ```True```
        * - container |rarr| elasticsearch
          - elasticsearch container image
          - str
          - ```docker.elastic.co/elasticsearch/elasticsearch:<version>```
        * - container |rarr| kibana
          - kibana container image
          - str
          - ```docker.elastic.co/kibana/kibana:<version>```
        * - container |rarr| fluentd
          - fluentd container image
          - str
          - ```fluent/fluentd-kubernetes-daemonset:<version>```
        * - kubernetes |rarr| volumes |rarr| elasticsearch-data
          - Elasticsearch Kubernetes data volume
          - Mapping, :class:`KData_Value`, :class:`KData_ConfigMap`, :class:`KData_Secret`
          -
        * - kubernetes |rarr| resources |rarr| elasticsearch-statefulset
          - Elasticsearch Kubernetes StatefulSet resources
          - Mapping
          -
        * - kubernetes |rarr| resources |rarr| kibana-deployment
          - Kibana Kubernetes Deployment resources
          - Mapping
          -
        * - kubernetes |rarr| resources |rarr| fluentd-daemonset
          - Fluentd Kubernetes DaemonSet resources
          - Mapping
          -
    """
    def define_options(self) -> Optional[Any]:
        """
        Declare the options for the EFK builder.

        :return: The supported options
        """
        return {
            'basename': OptionDef(required=True, default_value='efk', allowed_types=[str]),
            'namespace': OptionDef(required=True, default_value='default', allowed_types=[str]),
            'config': {
                'elasticsearch_replicas': OptionDef(required=True, default_value=3, allowed_types=[int]),
                'probes': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                'kibana_service_port': OptionDef(required=True, default_value=80, allowed_types=[int]),
                'authorization': {
                    'serviceaccount_create': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                    'serviceaccount_use': OptionDef(allowed_types=[str]),
                    'roles_create': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                    'roles_bind': OptionDef(required=True, default_value=True, allowed_types=[bool]),
                },
            },
            'enable': {
                'kibana': OptionDef(required=True, default_value=True, allowed_types=[bool]),
            },
            'container': {
                'elasticsearch': OptionDef(required=True, default_value='docker.elastic.co/elasticsearch/elasticsearch:7.9.3', allowed_types=[str]),
                'kibana': OptionDef(required=True, default_value='docker.elastic.co/kibana/kibana:7.9.3', allowed_types=[str]),
                'fluentd': OptionDef(required=True, default_value='fluent/fluentd-kubernetes-daemonset:v1.11.4-debian-elasticsearch7-1.0', allowed_types=[str]),
            },
            'kubernetes': {
                'volumes': {
                    'elasticsearch-data': OptionDef(required=True, format=OptionDefFormat.KDATA_VOLUME,
                                                    allowed_types=[Mapping, *KDataHelper_Volume.allowed_kdata()]),
                },
                'resources': {
                    'elasticsearch-statefulset': OptionDef(allowed_types=[Mapping]),
                    'kibana-deployment': OptionDef(allowed_types=[Mapping]),
                    'fluentd-daemonset': OptionDef(allowed_types=[Mapping]),
                }
            },
        }



from typing import Optional, Sequence

from kubragen import KubraGen
from kubragen.builder import Builder
from kubragen.consts import PROVIDER_K3D, PROVIDER_K3S
from kubragen.data import ValueData
from kubragen.exception import InvalidParamError, InvalidNameError
from kubragen.helper import QuotedStr
from kubragen.kdatahelper import KDataHelper_Volume
from kubragen.object import ObjectItem, Object
from kubragen.types import TBuild, TBuildItem

from .option import EFKOptions


class EFKBuilder(Builder):
    """
    EFK builder.

    Based on `How To Set Up an Elasticsearch, Fluentd and Kibana (EFK) Logging Stack on Kubernetes <https://www.digitalocean.com/community/tutorials/how-to-set-up-an-elasticsearch-fluentd-and-kibana-efk-logging-stack-on-kubernetes>`_.

    .. list-table::
        :header-rows: 1

        * - build
          - description
        * - BUILD_ACCESSCONTROL
          - creates service account, roles, and roles bindings
        * - BUILD_CONFIG
          - creates configurations
        * - BUILD_SERVICE
          - creates StatefulSet and Services

    .. list-table::
        :header-rows: 1

        * - build item
          - description
        * - BUILDITEM_SERVICE_ACCOUNT
          - ServiceAccount
        * - BUILDITEM_ELASTICSEARCH_SERVICE
          - Elasticsearch Service
        * - BUILDITEM_ELASTICSEARCH_STATEFULSET
          - Elasticsearch StatefulSet
        * - BUILDITEM_KIBANA_DEPLOYMENT
          - Kibana Deployment
        * - BUILDITEM_KIBANA_SERVICE
          - Kibana Service
        * - BUILDITEM_FLUENTD_CLUSTER_ROLE
          - Fluentd Cluster Role
        * - BUILDITEM_FLUENTD_CLUSTER_ROLE_BINDING
          - Fluentd Cluster Role Binding
        * - BUILDITEM_FLUENTD_DAEMONSET
          - Fluentd Daemonset

    .. list-table::
        :header-rows: 1

        * - object name
          - description
          - default value
        * - service-account
          - ServiceAccount
          - ```<basename>```
        * - elasticsearch-service
          - Elasticsearch Service
          - ```<basename>-elasticsearch```
        * - elasticsearch-statefulset
          - Elasticsearch StatefulSet
          - ```<basename>-elasticsearch```
        * - elasticsearch-pod-label-all
          - Elasticsearch label *app* to be used by selection
          - ```<basename>-elasticsearch```
        * - kibana-service
          - Kibana Service
          - ```<basename>-kibana```
        * - kibana-deployment
          - Kibana Deployment
          - ```<basename>-kibana```
        * - kibana-pod-label-all
          - Kibana label *app* to be used by selection
          - ```<basename>-kibana```
        * - fluentd-cluster-role
          - Fluentd ClusterRole
          - ```<basename>-fluentd```
        * - fluentd-cluster-role-binding
          - Fluentd ClusterRoleBinding
          - ```<basename>-fluentd```
        * - fluentd-deployment
          - Fluentd Deployment
          - ```<basename>-fluentd```
        * - fluentd-pod-label-all
          - Fluentd label *app* to be used by selection
          - ```<basename>-fluentd```
    """
    options: EFKOptions
    configfile: Optional[str]
    _namespace: str

    SOURCE_NAME = 'kg_efk'

    BUILD_ACCESSCONTROL = TBuild('accesscontrol')
    BUILD_CONFIG = TBuild('config')
    BUILD_SERVICE = TBuild('service')

    BUILDITEM_SERVICE_ACCOUNT = TBuildItem('service-account')
    BUILDITEM_ELASTICSEARCH_SERVICE = TBuildItem('elasticsearch-service')
    BUILDITEM_ELASTICSEARCH_STATEFULSET = TBuildItem('elasticsearch-statefulset')
    BUILDITEM_KIBANA_DEPLOYMENT = TBuildItem('kibana-deployment')
    BUILDITEM_KIBANA_SERVICE = TBuildItem('kibana-service')
    BUILDITEM_FLUENTD_CLUSTER_ROLE = TBuildItem('fluentd-cluster-role')
    BUILDITEM_FLUENTD_CLUSTER_ROLE_BINDING = TBuildItem('fluentd-cluster-role-binding')
    BUILDITEM_FLUENTD_DAEMONSET = TBuildItem('fluentd-daemonset')

    def __init__(self, kubragen: KubraGen, options: Optional[EFKOptions] = None):
        super().__init__(kubragen)
        if options is None:
            options = EFKOptions()
        self.options = options
        self.configfile = None

        self._namespace = self.option_get('namespace')

        if self.option_get('config.authorization.serviceaccount_create') is not False:
            serviceaccount_name = self.basename()
        else:
            serviceaccount_name = self.option_get('config.authorization.serviceaccount_use')
            if serviceaccount_name == '':
                serviceaccount_name = None

        if self.option_get('config.authorization.roles_bind') is not False:
            if serviceaccount_name is None:
                raise InvalidParamError('To bind roles a service account is required')

        self.object_names_init({
            'service-account': serviceaccount_name,
            'elasticsearch-service': self.basename('-elasticsearch'),
            'elasticsearch-statefulset': self.basename('-elasticsearch'),
            'elasticsearch-pod-label-app': self.basename('-elasticsearch'),
            'fluentd-cluster-role': self.basename('-fluentd'),
            'fluentd-cluster-role-binding': self.basename('-fluentd'),
            'fluentd-daemonset': self.basename('-fluentd'),
            'fluentd-pod-label-app': self.basename('-fluentd'),
        })
        if self.option_get('enable.kibana'):
            self.object_names_init({
                'kibana-service': self.basename('-kibana'),
                'kibana-deployment': self.basename('-kibana'),
                'kibana-pod-label-app': self.basename('-kibana'),
            })

    def option_get(self, name: str):
        return self.kubragen.option_root_get(self.options, name)

    def basename(self, suffix: str = ''):
        return '{}{}'.format(self.option_get('basename'), suffix)

    def namespace(self):
        return self._namespace

    def build_names(self) -> Sequence[TBuild]:
        return [self.BUILD_ACCESSCONTROL, self.BUILD_CONFIG, self.BUILD_SERVICE]

    def build_names_required(self) -> Sequence[TBuild]:
        ret = [self.BUILD_SERVICE]
        if self.option_get('config.authorization.serviceaccount_create') is not False or \
                self.option_get('config.authorization.roles_create') is not False:
            ret.append(self.BUILD_ACCESSCONTROL)
        return ret

    def builditem_names(self) -> Sequence[TBuildItem]:
        return [
            self.BUILDITEM_SERVICE_ACCOUNT,
            self.BUILDITEM_ELASTICSEARCH_SERVICE,
            self.BUILDITEM_ELASTICSEARCH_STATEFULSET,
            self.BUILDITEM_KIBANA_DEPLOYMENT,
            self.BUILDITEM_KIBANA_SERVICE,
            self.BUILDITEM_FLUENTD_CLUSTER_ROLE,
            self.BUILDITEM_FLUENTD_CLUSTER_ROLE_BINDING,
            self.BUILDITEM_FLUENTD_DAEMONSET,
        ]

    def internal_build(self, buildname: TBuild) -> Sequence[ObjectItem]:
        if buildname == self.BUILD_ACCESSCONTROL:
            return self.internal_build_accesscontrol()
        elif buildname == self.BUILD_CONFIG:
            return self.internal_build_config()
        elif buildname == self.BUILD_SERVICE:
            return self.internal_build_service()
        else:
            raise InvalidNameError('Invalid build name: "{}"'.format(buildname))

    def internal_build_accesscontrol(self) -> Sequence[ObjectItem]:
        ret = []

        if self.option_get('config.authorization.serviceaccount_create') is not False:
            ret.extend([
                Object({
                    'apiVersion': 'v1',
                    'kind': 'ServiceAccount',
                    'metadata': {
                        'name': self.object_name('service-account'),
                        'namespace': self.namespace(),
                    }
                }, name=self.BUILDITEM_SERVICE_ACCOUNT, source=self.SOURCE_NAME, instance=self.basename()),
            ])

        if self.option_get('config.authorization.roles_create') is not False:
            ret.extend([
                Object({
                    'apiVersion': 'rbac.authorization.k8s.io/v1',
                    'kind': 'ClusterRole',
                    'metadata': {
                        'name': self.object_name('fluentd-cluster-role'),
                    },
                    'rules': [{
                        'apiGroups': [''],
                        'resources': ['pods', 'namespaces'],
                        'verbs': ['get', 'list', 'watch']
                    }]
                }, name=self.BUILDITEM_FLUENTD_CLUSTER_ROLE, source=self.SOURCE_NAME, instance=self.basename()),
            ])

        if self.option_get('config.authorization.roles_bind') is not False:
            ret.extend([
                Object({
                    'kind': 'ClusterRoleBinding',
                    'apiVersion': 'rbac.authorization.k8s.io/v1',
                    'metadata': {
                        'name': self.object_name('fluentd-cluster-role-binding'),
                    },
                    'roleRef': {
                        'kind': 'ClusterRole',
                        'name': self.object_name('fluentd-cluster-role'),
                        'apiGroup': 'rbac.authorization.k8s.io'
                    },
                    'subjects': [{
                        'kind': 'ServiceAccount',
                        'name': self.object_name('service-account'),
                        'namespace': self.namespace(),
                    }]
                }, name=self.BUILDITEM_FLUENTD_CLUSTER_ROLE_BINDING, source=self.SOURCE_NAME, instance=self.basename())
            ])

        return ret

    def internal_build_config(self) -> Sequence[ObjectItem]:
        # Reserve for future use
        return []

    def internal_build_service(self) -> Sequence[ObjectItem]:
        ret = []

        ret.extend([
            Object({
                'kind': 'Service',
                'apiVersion': 'v1',
                'metadata': {
                    'name': self.object_name('elasticsearch-service'),
                    'namespace': self.namespace(),
                    'labels': {
                        'app': self.object_name('elasticsearch-pod-label-app'),
                    }
                },
                'spec': {
                    'selector': {
                        'app': self.object_name('elasticsearch-pod-label-app'),
                    },
                    'clusterIP': 'None',
                    'ports': [{
                        'port': 9200,
                        'name': 'rest'
                    }, {
                        'port': 9300,
                        'name': 'inter-node'
                    }],
                }
            }, name=self.BUILDITEM_ELASTICSEARCH_SERVICE, source=self.SOURCE_NAME, instance=self.basename()),
            Object({
                'apiVersion': 'apps/v1',
                'kind': 'StatefulSet',
                'metadata': {
                    'name': self.object_name('elasticsearch-statefulset'),
                    'namespace': self.namespace(),
                },
                'spec': {
                    'serviceName': self.object_name('elasticsearch-service'),
                    'replicas': self.option_get('config.elasticsearch_replicas'),
                    'selector': {
                        'matchLabels': {
                            'app': self.object_name('elasticsearch-pod-label-app'),
                        }
                    },
                    'template': {
                        'metadata': {
                            'labels': {
                                'app': self.object_name('elasticsearch-pod-label-app'),
                            }
                        },
                        'spec': {
                            'volumes': [
                                KDataHelper_Volume.info(base_value={
                                    'name': 'data',
                                }, value=self.option_get('kubernetes.volumes.elasticsearch-data')),
                            ],
                            'containers': [{
                                'name': 'elasticsearch',
                                'image': self.option_get('container.elasticsearch'),
                                'ports': [{
                                    'containerPort': 9200,
                                    'name': 'rest',
                                    'protocol': 'TCP'
                                },
                                {
                                    'containerPort': 9300,
                                    'name': 'inter-node',
                                    'protocol': 'TCP'
                                }],
                                'volumeMounts': [{
                                    'name': 'data',
                                    'mountPath': '/usr/share/elasticsearch/data'
                                }],
                                'env': [{
                                    'name': 'cluster.name',
                                    'value': self.object_name('elasticsearch-statefulset'),
                                },
                                {
                                    'name': 'NODE_NAME',
                                    'valueFrom': {
                                        'fieldRef': {
                                            'fieldPath': 'metadata.name'
                                        }
                                    },
                                },
                                {
                                    'name': 'node.name',
                                    'value': QuotedStr('$(NODE_NAME).{}'.format(self.object_name('elasticsearch-service'))),
                                },
                                {
                                    'name': 'discovery.seed_hosts',
                                    'value': ','.join(['{}-{}.{}'.format(
                                        self.object_name('elasticsearch-statefulset'), rpl, self.object_name('elasticsearch-service'))
                                        for rpl in range(self.option_get('config.elasticsearch_replicas'))
                                    ]),
                                },
                                {
                                    'name': 'cluster.initial_master_nodes',
                                    'value': ','.join(['{}-{}.{}'.format(
                                        self.object_name('elasticsearch-statefulset'), rpl, self.object_name('elasticsearch-service'))
                                        for rpl in range(self.option_get('config.elasticsearch_replicas'))
                                    ]),
                                },
                                {
                                    'name': 'ES_JAVA_OPTS',
                                    'value': '-Xms512m '
                                             '-Xmx512m'
                                }],
                                'resources': ValueData(value=self.option_get('kubernetes.resources.elasticsearch-statefulset'),
                                                       disabled_if_none=True),
                            }],
                            'initContainers': [{
                                'name': 'fix-permissions',
                                'image': 'busybox',
                                'command': [
                                    'sh',
                                    '-c',
                                    'chown -R '
                                    '1000:1000 '
                                    '/usr/share/elasticsearch/data'
                                ],
                                'securityContext': {
                                    'privileged': True
                                },
                                'volumeMounts': [{
                                    'name': 'data',
                                    'mountPath': '/usr/share/elasticsearch/data'
                                }],
                            },
                            {
                                'name': 'increase-vm-max-map',
                                'image': 'busybox',
                                'command': [
                                    'sysctl',
                                    '-w',
                                    'vm.max_map_count=262144'
                                ],
                                'securityContext': {
                                    'privileged': True
                                },
                            },
                            {
                                'name': 'increase-fd-ulimit',
                                'image': 'busybox',
                                'command': [
                                    'sh',
                                    '-c',
                                    'ulimit -n '
                                    '65536'
                                ],
                                'securityContext': {
                                    'privileged': True
                                },
                            }]
                        }
                    },
                }
            }, name=self.BUILDITEM_ELASTICSEARCH_STATEFULSET, source=self.SOURCE_NAME, instance=self.basename()),
            Object({
                'apiVersion': 'apps/v1',
                'kind': 'DaemonSet',
                'metadata': {
                    'name': self.object_name('fluentd-daemonset'),
                    'namespace': self.namespace(),
                    'labels': {
                        'app': self.object_name('fluentd-pod-label-app'),
                    }
                },
                'spec': {
                    'selector': {
                        'matchLabels': {
                            'app': self.object_name('fluentd-pod-label-app'),
                        }
                    },
                    'template': {
                        'metadata': {
                            'labels': {
                                'app': self.object_name('fluentd-pod-label-app'),
                            }
                        },
                        'spec': {
                            'serviceAccount': self.object_name('service-account'),
                            'serviceAccountName': self.object_name('service-account'),
                            'tolerations': [{
                                'key': 'node-role.kubernetes.io/master',
                                'effect': 'NoSchedule'
                            }],
                            'containers': [{
                                'name': 'fluentd',
                                'image': self.option_get('container.fluentd'),
                                'env': [{
                                    'name': 'FLUENT_ELASTICSEARCH_HOST',
                                    'value': '{}.{}.svc.cluster.local'.format(self.object_name('elasticsearch-service'), self.namespace()),
                                },
                                {
                                    'name': 'FLUENT_ELASTICSEARCH_PORT',
                                    'value': '9200'
                                },
                                {
                                    'name': 'FLUENT_ELASTICSEARCH_SCHEME',
                                    'value': 'http'
                                },
                                {
                                    'name': 'FLUENTD_SYSTEMD_CONF',
                                    'value': 'disable'
                                },
                                ValueData(value={
                                    'name': 'FLUENT_CONTAINER_TAIL_PARSER_TYPE',
                                    'value': '/^(?<time>.+) (?<stream>stdout|stderr) [^ ]* (?<log>.*)$/',
                                }, enabled=self.kubragen.provider.provider == PROVIDER_K3D or self.kubragen.provider.provider == PROVIDER_K3S)],
                                'volumeMounts': [{
                                    'name': 'varlog',
                                    'mountPath': '/var/log'
                                },
                                {
                                    'name': 'varlibdockercontainers',
                                    'mountPath': '/var/lib/docker/containers',
                                    'readOnly': True
                                }],
                                'resources': ValueData(value=self.option_get('kubernetes.resources.fluentd-daemonset'),
                                                       disabled_if_none=True),

                            }],
                            'terminationGracePeriodSeconds': 30,
                            'volumes': [{
                                'name': 'varlog',
                                'hostPath': {
                                    'path': '/var/log'
                                }
                            },
                            {
                                'name': 'varlibdockercontainers',
                                'hostPath': {
                                    'path': '/var/lib/docker/containers'
                                }
                            }]
                        }
                    }
                }
            }, name=self.BUILDITEM_FLUENTD_DAEMONSET, source=self.SOURCE_NAME, instance=self.basename()),
        ])

        if self.option_get('enable.kibana'):
            ret.extend([
                Object({
                    'apiVersion': 'v1',
                    'kind': 'Service',
                    'metadata': {
                        'name': self.object_name('kibana-service'),
                        'namespace': self.namespace(),
                        'labels': {
                            'app': self.object_name('kibana-pod-label-app'),
                        },
                    },
                    'spec': {
                        'ports': [{
                            'port': self.option_get('config.kibana_service_port'),
                            'targetPort': 5601,
                        }],
                        'selector': {
                            'app': self.object_name('kibana-pod-label-app'),
                        }
                    }
                }, name=self.BUILDITEM_KIBANA_SERVICE, source=self.SOURCE_NAME, instance=self.basename()),
                Object({
                    'apiVersion': 'apps/v1',
                    'kind': 'Deployment',
                    'metadata': {
                        'name': self.object_name('kibana-deployment'),
                        'namespace': self.namespace(),
                        'labels': {
                            'app': self.object_name('kibana-pod-label-app'),
                        }
                    },
                    'spec': {
                        # 'replicas': 1,
                        'selector': {
                            'matchLabels': {
                                'app': self.object_name('kibana-pod-label-app'),
                            }
                        },
                        'template': {
                            'metadata': {
                                'labels': {
                                    'app': self.object_name('kibana-pod-label-app'),
                                }
                            },
                            'spec': {
                                'containers': [{
                                    'name': 'kibana',
                                    'image': self.option_get('container.kibana'),
                                    'env': [{
                                        'name': 'ELASTICSEARCH_HOSTS',
                                        'value': 'http://{}:9200'.format(self.object_name('elasticsearch-service')),
                                    }],
                                    'ports': [{
                                        'containerPort': 5601
                                    }],
                                    'livenessProbe': ValueData(value={
                                        'httpGet': {
                                            'path': '/api/status',
                                            'port': 5601,
                                        },
                                        'initialDelaySeconds': 30,
                                        'timeoutSeconds': 20,
                                    }, enabled=self.option_get('config.probes')),
                                    'readinessProbe': ValueData(value={
                                        'httpGet': {
                                            'path': '/api/status',
                                            'port': 5601,
                                        },
                                        'initialDelaySeconds': 30,
                                        'timeoutSeconds': 20,
                                    }, enabled=self.option_get('config.probes')),
                                    'resources': ValueData(
                                        value=self.option_get('kubernetes.resources.kibana-deployment'),
                                        disabled_if_none=True),
                                }]
                            }
                        }
                    }
                }, name=self.BUILDITEM_KIBANA_DEPLOYMENT, source=self.SOURCE_NAME, instance=self.basename()),
            ])

        return ret



import unittest

from kubragen import KubraGen
from kubragen.jsonpatch import FilterJSONPatches_Apply, ObjectFilter, FilterJSONPatch
from kubragen.provider import Provider_Generic

from kg_efk import EFKBuilder, EFKOptions


class TestBuilder(unittest.TestCase):
    def setUp(self):
        self.kg = KubraGen(provider=Provider_Generic())

    def test_empty(self):
        efk_config = EFKBuilder(kubragen=self.kg, options=EFKOptions({
            'kubernetes': {
                'volumes': {
                    'elasticsearch-data': {
                        'emptyDir': {},
                    }
                }
            }
        }))
        self.assertEqual(efk_config.object_name('elasticsearch-service'), 'efk-elasticsearch')
        self.assertEqual(efk_config.object_name('fluentd-daemonset'), 'efk-fluentd')

    def test_basedata(self):
        efk_config = EFKBuilder(kubragen=self.kg, options=EFKOptions({
            'namespace': 'myns',
            'basename': 'myefk',
            'kubernetes': {
                'volumes': {
                    'elasticsearch-data': {
                        'emptyDir': {},
                    }
                }
            }
        }))

        self.assertEqual(efk_config.object_name('elasticsearch-service'), 'myefk-elasticsearch')
        self.assertEqual(efk_config.object_name('fluentd-daemonset'), 'myefk-fluentd')

        FilterJSONPatches_Apply(items=efk_config.build(efk_config.BUILD_SERVICE), jsonpatches=[
            FilterJSONPatch(filters=ObjectFilter(names=[efk_config.BUILDITEM_ELASTICSEARCH_SERVICE]), patches=[
                {'op': 'check', 'path': '/metadata/name', 'cmp': 'equals', 'value': 'myefk-elasticsearch'},
                {'op': 'check', 'path': '/metadata/namespace', 'cmp': 'equals', 'value': 'myns'},
            ]),
        ])
